#ifndef carto_h
#define carto_h

/* $Id: carto.h,v 1.9 2013/09/18 03:01:47 dkl Exp $ */ 

/* carto - cartographic stuff */


extern "C" {

#define PROJ_4_8

#ifdef PROJ_4_8
    #include <proj_api.h>
#else
    #include <projects.h>
#endif
};


//
//  Geodetic Constants
//
//  Earth radius in meters
#define EARTH_RADIUS 6378100
//
//  Earth circumference in meters
#define EARTH_CIRCUMFERENCE 40074784
//
//  One 360th of the earth's circumference in meters
#define ONE_DEGREE_ARC 111318.845
//
#define SIXTIETH (1.0 / 60.0)
#define THIRTYSIXHUNDRETH (SIXTIETH / 60.0)


#define UTM_NAME "UTM"
#define AEA_NAME "AEA"
#define LATLONG_NAME "LATLONG"
#define STATEPLANE_NAME "STATEPLANE"
#define WGS_NAME "WGS"

enum ProjType { UTM, AEA, LATLONG, STATEPLANE, WGS, NUM_PROJECTIONS, NO_PROJ };
enum DatumType { NAD27, NAD83, NUM_DATUMS, NO_DATUM};
enum EllipsType { CLRK66, GRS80, NUM_ELLIPSOIDS, NO_ELLIPS};

//char * theProjNames { "UTM", "AEA", "LATLONG", "STATEPLANE", "WGS"};
//char * theDatumNames { "NAD27", "NAD83"};
//char * theEllipseNames { "CLRK66", "GRS80" };

//enum ProjNames { "UTM", "AEA", "LATLONG", "STATEPLANE", "WGS"};
//enum DatumNames { "NAD27", "NAD83"};
//enum EllipseNames { "CLRK66", "GRS80" };

#define DEFAULT_PROJECTION AEA
#define DEFAULT_DATUM NAD27
#define DEFAULT_ELLIPSOID CLRK66



class degree {
  public:
	degree (double);
	degree (int, int, int);
	double 	degrees;	
	int		deg;
	int		min;
	int 		sec;	
};

class carto { 
   public:
	   carto ();
	   carto (char *);
	   carto (char *, int );
	   carto (char *, char *);
		carto (char *, int , int );
		carto (const char *nm=NULL, int pr=0, int dat=0, int ell=0);
		carto (char *, int , char *, char *);
	   carto (carto *);
	   ~carto ();

	   void 					SetMeters (int , int );
	   void 					SetLatLong (double, double);
	   double 				GetEW ();
	   double 				GetNS ();
	   int					GetZone();
	   int 					FindArg (char *);
	   int 					AddArg (char *);
	   int 					SetParam (char *);
	   void 					SetProjection (char *);
   
	   void 					Open();
	   projPJ 					GetPJ();
  		 
	   static void 		ListProjections ();
	   static void 		ListDatums ();
	   static void 		ListEllipsoids ();

	   static const char * 		ProjName (ProjType);
	   static const char * 		DatumName (DatumType);
	   static const char * 		EllipsName (EllipsType);

	   static int 			ValidateProjection (char *);
	   static int 			ValidateDatum (char *);
	   static int 			ValidateEllipsoid (char *);

	   static double 		GetLatFromName (char *);
	   static double 		GetLongFromName (char *);
	   static char *		GetTileNameFromLatLong (double , double );
	   static char *		GetQuadNameFromLatLong (double , double );
   
	   static int			GetZoneOf (char *);
	   static int			GetZoneOf (double);

		enum 	ProjType 	projections;
		enum 	DatumType 	datums;
		enum 	EllipsType 	ellipsoids;

      char 	*		projGivenName;
      char 	*		datumGivenName;
      char 	*		ellipsGivenName;

	   static int			debug;
	   char *				name;
   
   private:
	   projPJ 				pj;		// the projection object 
	   projUV 				p;
	   int 					proj, datum, ellips;
	   int 					zone; 		// the zone, if UTM
   
	   int 					numArgs;
	   //char 					*args[];
	   char 					**args;

};

#endif 
